!!ARBfp1.0

# Displacement Filter by James A. McCombe, Mon Jul 14 2003
#
# Implements per-pixel lighting effect.  A light position in is converted into model space
# by the vertex program and passed into the fragment program through a texture coordinate
# channel.  The incoming light direction vector is calculated from the light position and fragment
# position.  The reflected light vector is calculated using the fragment normal at that position, 
# acquired from texture unit 1.  In order to get the light intensity at this point, the outgoing
# light direction vector (after reflection) is DOT3'd with the light direction vector which, for
# simplicity is pointing directly along the surface normal.
#
# With the light intensity known, it is attenuated by multiplying it with a function of the 
# distance of the light source from the fragment position which was calculated near the beginning
# of the program.
#
# Finally, its modulated with the light brightness and color parameter and modulated with the base
# texture map stored in texture unit 0.
#
# The comments in the program are intentionally verbose to as to better document this process.
#
# Texture units:
#   Unit 0 - Base texture map
#   Unit 1 - Surface normals, encoded in the red, green and blue color channels
#
# Parameters:
#   These are all accessible from the vertex program tab.

# Main contributing entities
PARAM  difLightColorAndBrightness = program.env[0];

PARAM  ambLightColorAndBrightness = program.env[1];
PARAM  lightDir        	= program.env[2];

ATTRIB fragmentPosition         = fragment.texcoord[2];   # Model space
TEMP fragmentNormal;
ATTRIB polygonNormal            = fragment.texcoord[1];
ATTRIB baseTexCoords            = fragment.texcoord[0];
ATTRIB tangent            = fragment.texcoord[3];
ATTRIB bitangent            = fragment.texcoord[4];
ATTRIB base2TexCoords            = fragment.texcoord[5];

# Basic constants used in calculations
PARAM  zero             = { 0.0, 0.0, 0.0, 0.0 };
PARAM  one              = { 1.0, 1.0, 1.0, 1.0 };
PARAM  constants        = { 0.5, 5.0, 0.0, 0.0 };

TEMP distanceFromLightToFragment;
TEMP lightDirection; # Always negative normal in this example

TEMP t0, t1, t2;	 # General purpose temporaries

# Calculate the vector from light to fragment
#MOV t0, zero;        # Initialise t0 to all zeros
#SUB t0.xyz, lightPosition, -fragmentPosition;
#DP3 t0.w, t0, t0;
#RSQ t0.w, t0.w;

# Calculate the distance from light to fragment (used for attenuation later)
#RCP distanceFromLightToFragment, t0.w;
#MUL distanceFromLightToFragment, distanceFromLightToFragment, distanceFromLightToFragment;
#SUB distanceFromLightToFragment, one, distanceFromLightToFragment;
#MUL t0.xyz, t0, t0.w;    # t0 now contains the normalized 
						 # incoming light vector for each fragment

MOV t0,lightDir;

# Acquire the fragment normal (from the normal map texture)
TEX fragmentNormal, fragment.texcoord[0], texture[2], 2D;
SUB fragmentNormal.xy, fragmentNormal, constants.x;

TEMP normal;
MUL normal, polygonNormal,fragmentNormal.b;
MAD normal, tangent,-fragmentNormal.g,normal;
MAD normal, bitangent, -fragmentNormal.r,normal;
#MOV normal, polygonNormal;

# Calculate the light vector after reflection
XPD t1, normal, t0;
XPD t1, normal, t1;
ADD t2, t0, t1;
ADD t0, t2, t1;

# Dot product incident light ray with light direction vector
MOV lightDirection, normal;
DP3 t0, t0, lightDirection;

# Attenuate light intensity scalar with function of light distance
MUL distanceFromLightToFragment, distanceFromLightToFragment,5;
#MUL_SAT t0, t0, distanceFromLightToFragment;

# Modulate with light brightness
#MUL t0, t0, difLightColorAndBrightness.w;
#MUL t0, t0, constants.y;
MUL t0.rgb, t0, difLightColorAndBrightness;
MAD t0,fragmentNormal.b, ambLightColorAndBrightness,t0;
MIN t0,t0,one;

# Modulate with base texture
TEX t1, baseTexCoords, texture[0], 2D;
TEX t2, base2TexCoords, texture[1], 2D;
MUL t1,t1,t2;
#ADD t1, t1, t0;
MUL t0, t1, t0;

# Write the output color
MOV result.color, t0;
MOV result.color.a, 1.0;

END